(* -----------------------------------------------------------------------------
  Create Thumbnail With IEConvertToThumbnail
  Copyright  2015   : Copyright Adirondack Software & Graphics
  Last Modification  : 05-08-2015
  Source File        : Unit1.pas
  Compiler           : Delphi XE7
  Operating System   : Windows 8.1
  This file is copyright  W W Miller, 1986-2015.
  This demo will only run on Windows operating systems Vista or higher
  and requires Delphi 2010 or higher
  It may be used without restriction. This code distributed on an "AS IS"
  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
  --------------------------------------------------------------------------- *)
unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs,
  imageenview, ieview, iemview, imageenio, Vcl.ComCtrls, Vcl.StdCtrls,
  Vcl.ExtCtrls, Vcl.ExtDlgs;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Open1: TButton;
    Thumbnail1: TButton;
    StatusBar1: TStatusBar;
    ImageEnMView1: TImageEnMView;
    ImageEnView1: TImageEnView;
    OpenPictureDialog1: TOpenPictureDialog;
    Clear1: TButton;
    Stretch1: TCheckBox;
    AddBorder1: TCheckBox;
    BorderColor1: TColorBox;
    AddShadow1: TCheckBox;
    ShadowOffset1: TEdit;
    UpDown1: TUpDown;
    ShadowColor1: TColorBox;
    BackgroundColor1: TColorBox;
    Fit1: TCheckBox;
    SaveAs1: TButton;
    SavePictureDialog1: TSavePictureDialog;
    CreateAndSaveAll1: TButton;
    ImageEnView2: TImageEnView;
    Exit1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure Thumbnail1Click(Sender: TObject);
    procedure Clear1Click(Sender: TObject);
    procedure ImageEnMView1ImageSelect(Sender: TObject; idx: Integer);
    procedure Fit1Click(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure CreateAndSaveAll1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
  private
    { Private declarations }
    AFilename: string;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses Winapi.ShlObj, hyiedefs, hyieutils, imageenproc, iexHelperFunctions;

function DocumentsFolder: string;
{ Find Documents folder location. }
var
  iResult: bool;
  iPath: array [0 .. MAX_PATH] of char;
begin
  iResult := Winapi.ShlObj.ShGetSpecialFolderPath(0, iPath,
    CSIDL_PERSONAL, False);
  if not iResult then
    raise Exception.Create('Could not find Documents folder location.');
  Result := IncludeTrailingPathDelimiter(iPath);
end;

function DesktopFolder: string;
{ Find Desktop folder location. }
var
  iResult: bool;
  iPath: array [0 .. MAX_PATH] of char;
begin
  iResult := Winapi.ShlObj.ShGetSpecialFolderPath(0, iPath,
    CSIDL_DESKTOPDIRECTORY, False);
  if not iResult then
    raise Exception.Create('Could not find Documents folder location.');
  Result := IncludeTrailingPathDelimiter(iPath);
end;

function AddThousandSeparator(const AString: string; const AChar: char): string;
{ Add a specified thousand separator in the correct location in a string. }
var
  i: Integer; { Loop through separator position }
begin
  Result := AString;
  i := Length(AString) - 2;
  while i > 1 do
  begin
    Insert(AChar, Result, i);
    i := i - 3;
  end;
end;

function AddDefThousandSeparator(const AString: string): string;
{ Add the local default thousand separator in the correct location
  in a string. }
begin
  Result := AddThousandSeparator(AString,
    System.SysUtils.FormatSettings.ThousandSeparator);
end;

function IntegerToString(const i: Int64): string;
{ Add the local default thousand separator in the correct
  location in a number and returns a string. }
begin
  Result := AddThousandSeparator(IntToStr(i),
    System.SysUtils.FormatSettings.ThousandSeparator);
end;

function FilterIndex(const AFilter, AFilename: string): Integer;
{ Get the Filter index for the filename. }
var
  iList: TStringList;
  iBuffer: TStringList;
  i: Integer;
  iKey: string;
  iExt: string;
begin
  Result := -1;
  i := 0;
  iList := TStringList.Create;
  iList.LineBreak := '|';
  iList.Text := AFilter;
  iBuffer := TStringList.Create;
  iBuffer.Sorted := False;
  try
    while i < iList.Count do
    begin
      inc(i);
      iKey := AnsiLowerCase(iList[i]);
      inc(i);
      iBuffer.Add(iKey);
    end;
    iExt := AnsiLowerCase(ExtractFileExt(AFilename));
    for i := 0 to iBuffer.Count - 1 do
      if iBuffer[i] = '*' + iExt then
      begin
        Result := i + 1;
        exit;
      end;
  finally
    FreeAndNil(iBuffer);
    FreeAndNil(iList);
  end;
end;

function DelText(const ANeedle, AHaystack: string): string;
{ Deletes the first occurrence of text Needle from text HayStack and returns
  the resulting string. HayStack is returned unchanged if it does not contain
  Needle. The search is case insensitive. }
var
  iStartIndex: Integer;
begin
  Result := AHaystack;
  iStartIndex := System.SysUtils.AnsiPos(System.SysUtils.AnsiLowerCase(ANeedle),
    System.SysUtils.AnsiLowerCase(AHaystack));
  if iStartIndex > 0 then
    Delete(Result, iStartIndex, Length(ANeedle));
end;

function JustFilename(const APathName: string): string;
{ Return a filename from a string. }
var
  i: Integer;
  iString: string;
  { Turn- Reverse characters in a string ABCD -> DCBA }
  function Turn(const AString: string): string;
  var
    i: Integer;
  begin
    Result := '';
    if AString <> '' then
      for i := 1 to Length(AString) do
        Result := AString[i] + Result;
  end;

begin
  iString := Turn(APathName);
  i := Pos('\', iString);
  if i = 0 then
    i := Pos(':', iString);
  if i = 0 then
    Result := APathName
  else
    Result := Turn(Copy(iString, 1, i - 1));
end;

function JustName(const APathName: string): string;
{ Return just the name from a file string. }
var
  iString: string;
begin
  iString := JustFilename(APathName);
  if Pos('.', iString) <> 0 then
    Result := Copy(iString, 1, Pos('.', iString) - 1)
  else
    Result := iString;
end;

procedure TForm1.Clear1Click(Sender: TObject);
var
  i: Integer;
begin
  ImageEnMView1.Clear;
  ImageEnView1.Clear;
  ImageEnView2.Clear;
  for i := 0 to StatusBar1.Panels.Count - 1 do
    StatusBar1.Panels[i].Text := '';
end;

procedure TForm1.CreateAndSaveAll1Click(Sender: TObject);
{ Create thumbnails and save as jpg to Desktop folder and save full images as a
  tif file to Desktop folder }
var
  i: Integer;
  iIEBitmap: TIEBitmap;
  iFolder: string;
  iTifFilename: string;
  iThumbnailFilename: string;
begin
  for i := 0 to ImageEnMView1.ImageCount - 1 do
  begin
    iIEBitmap := ImageEnMView1.GetTIEBitmap(i);
    try
      { Save to the desktop folder }
      iFolder := DesktopFolder;
      { Create the tif file filename }
      iTifFilename := iFolder +
        JustName(ImageEnMView1.MIO.Params[i].FileName) + '.tif';
      { Create the jpg thumbnail filename }
      iThumbnailFilename := iFolder +
        JustName(ImageEnMView1.MIO.Params[i].FileName) + '_thumbnail.jpg';
      { Save all images stored in ImageEnMView as full size tif files }
      iIEBitmap.IESaveToFile(iTifFilename, ioTIFF);
      { Convert the full image to a thumbnail }
      iIEBitmap.IEConvertToThumbnail(150, 116, Stretch1.Checked, rfLanczos3,
        AddBorder1.Checked, BorderColor1.Selected, AddShadow1.Checked, 0,
        StrToIntDef(ShadowOffset1.Text, 4), ShadowColor1.Selected,
        BackgroundColor1.Selected);
      { Save all thumbnails as jpg }
      iIEBitmap.IESaveToFile(iThumbnailFilename, ioJPEG);
    finally
      ImageEnMView1.ReleaseBitmap(ImageEnMView1.SelectedImage);
    end;
  end;
end;

procedure TForm1.Exit1Click(Sender: TObject);
{ Exit to windows. }
begin
  Close;
end;

procedure TForm1.Fit1Click(Sender: TObject);
{ Fit the main image to the screen. }
begin
  if Fit1.Checked then
    ImageEnView1.Fit
  else
    ImageEnView1.Zoom := 100;
end;

procedure TForm1.FormCreate(Sender: TObject);
{ Register file formats for file dialogs, initialize ImageEnMView, initialize
  gestures and set cursors. }
begin
  IERegisterFormats;
  OpenPictureDialog1.Filter := GraphicFilter(TGraphic);
  SavePictureDialog1.Filter := GraphicFilter(TGraphic);
  ImageEnMView1.SetModernStyling(False, 150, 125);
  ImageEnMView1.Clear;
  ImageEnView2.Blank;
  ImageEnMView1.Gestures.Pan.Enabled := True;
  ImageEnView1.Gestures.Pan.Enabled := True;
  ImageEnView1.Gestures.Zoom.Enabled := True;
  ImageEnView1.Cursor := crIECross;
  ImageEnView2.Cursor := crDefault;
end;

procedure TForm1.FormDestroy(Sender: TObject);
{ Unregister graphic file formats. }
begin
  IEUnregisterFormats;
end;

procedure TForm1.ImageEnMView1ImageSelect(Sender: TObject; idx: Integer);
{ Display the selected image and its thumbnail. }
var
  iIEBitmap: TIEBitmap;
begin
  iIEBitmap := TIEBitmap.Create;
  try
    { Display the full size image selected with ImageENMView in ImageEnView1 }
    iIEBitmap.Assign(ImageEnMView1.GetTIEBitmap(idx));
    ImageEnView1.IEBitmap.Assign(iIEBitmap);
    ImageEnView1.Background := TRGB2TColor(ImageEnView1.IEBitmap.Pixels[0,
       ImageEnView1.IEBitmap.Height-1]);
    ImageEnView1.Update;
    { Convert the full image to a thumbnail }
    iIEBitmap.IEConvertToThumbnail(150, -1, Stretch1.Checked, rfLanczos3,
      AddBorder1.Checked, BorderColor1.Selected, AddShadow1.Checked, 0,
      StrToIntDef(ShadowOffset1.Text, 4), ShadowColor1.Selected,
      BackgroundColor1.Selected);
    { To display a thumbnail add a ImageEnView2 on top of ImageEn1 }
    { Copy the thumbnail to ImageEnView2 }
    ImageEnView2.IEBitmap.Assign(iIEBitmap);
    ImageEnView2.Background := TRGB2TColor(ImageEnView1.IEBitmap.Pixels[0,
       ImageEnView2.IEBitmap.Height-1]);
    ImageEnView2.Update;
    if Fit1.Checked then
      ImageEnView1.Fit;
    AFilename := ImageEnMView1.MIO.Params[idx].FileName;
    StatusBar1.Panels[0].Text :=
      ExtractFileDir(AFileName);
    StatusBar1.Panels[1].Text :=
      ExtractFileName(AFileName);
    StatusBar1.Panels[2].Text := 'Width: ' +
      IntegerToString(ImageEnMView1.MIO.Params[idx].Width);
    StatusBar1.Panels[3].Text := 'Height: ' +
      IntegerToString(ImageEnMView1.MIO.Params[idx].Height);
  finally
    ImageEnMView1.ReleaseBitmap(idx);
  end;
end;

procedure TForm1.Open1Click(Sender: TObject);
{ Open multiselected images and display in ImageEnMView. }
var
  i: Integer;
  iFilename: string;
begin
  OpenPictureDialog1.DefaultExt := 'jpg';
  if OpenPictureDialog1.Execute then
  begin
    for i := 0 to OpenPictureDialog1.Files.Count - 1 do
    begin
      iFilename := OpenPictureDialog1.Files[i];
      ImageEnMView1.MIO.LoadFromFile(iFilename);
      ImageEnMView1.Update;
    end;
    ImageEnMView1.SelectedImage := 0;
  end;
end;

procedure TForm1.SaveAs1Click(Sender: TObject);
var
  iFolder: string;
  iExtension: string;
  iIndex: Integer;
  iTifFilename: string;
  iThumbnailFilename: string;
begin
  iExtension := DelText('.', ExtractFileExt(AFilename));
  SavePictureDialog1.FileName := ExtractFileName(AFilename);
  SavePictureDialog1.DefaultExt := iExtension;
  SavePictureDialog1.Title := 'SaveAs...';
  { Get the filter index of the filename }
  iIndex := FilterIndex(SavePictureDialog1.Filter, AFilename);
  { Set the filter index for the selected file format }
  SavePictureDialog1.FilterIndex := iIndex;
  if SavePictureDialog1.Execute then
  begin
    iFolder := IncludeTrailingPathDelimiter
      (ExtractFileDir(SavePictureDialog1.FileName));
    { Create the tif file filename }
    iTifFilename := iFolder + JustName(SavePictureDialog1.FileName) + '.tif';
    { Create the jpg thumbnail filename }
    iThumbnailFilename := iFolder + JustName(SavePictureDialog1.FileName) +
      '_thumbnail.jpg';
    { Save the selected ImageEnMView image as a full image tif file }
    ImageEnMView1.MIO.SaveToFile(iTifFilename, True);
    { Save the jpg thumbnail }
    ImageEnView1.IO.SaveToFile(iThumbnailFilename);
  end;
end;

procedure TForm1.Thumbnail1Click(Sender: TObject);
{ Create a thumbnail. }
var
  iIEBitmap: TIEBitmap;
begin
  ImageEnView1.Zoom := 100;
  { The full tif image is stored in the IMageEnMView Frame }
  iIEBitmap := ImageEnMView1.GetTIEBitmap(ImageEnMView1.SelectedImage);
  try
    { Convert the full image to a thumbnail }
    iIEBitmap.IEConvertToThumbnail(150, 116, Stretch1.Checked, rfLanczos3,
      AddBorder1.Checked, BorderColor1.Selected, AddShadow1.Checked, 0,
      StrToIntDef(ShadowOffset1.Text, 4), ShadowColor1.Selected,
      BackgroundColor1.Selected);
    { The jpeg thumbnail is stored in ImageEnView so we copy the full frame to ImageEnView }
    ImageEnView1.IEBitmap.Assign(iIEBitmap);
    ImageEnView1.IO.Params.Assign(ImageEnMView1.MIO.Params
      [ImageEnMView1.SelectedImage]);
    { We changed the dimensions of the image to a thumbnail so store the
      appropriate IO.Params }
    ImageEnView1.IO.Params.Width := iIEBitmap.Width;
    ImageEnView1.IO.Params.Height := iIEBitmap.Height;
    ImageEnView1.IO.Params.FileType := ioJPEG;
    ImageEnView1.Update;
    StatusBar1.Panels[2].Text := 'Width: ' +
      IntegerToString(ImageEnView1.IO.Params.Width);
    StatusBar1.Panels[3].Text := 'Height: ' +
      IntegerToString(ImageEnView1.IO.Params.Height);
  finally
    ImageEnMView1.ReleaseBitmap(ImageEnMView1.SelectedImage);
  end;
end;

end.
